@@ -5,12 +5,13 @@ All Rights Reserved.
 NOTICE: Adobe permits you to use, modify, and distribute this file in accordance
 with the terms of the Adobe license agreement accompanying it.
 """
-
+import json
+from json import JSONDecodeError
 from typing import Any, Dict, Set
 
 import sqlalchemy
 from pydantic import BaseModel  # pylint: disable=no-name-in-module
-from pydantic.error_wrappers import ValidationError
+from pydantic.error_wrappers import ValidationError, ErrorWrapper
 
 from .mappers import DbMapResultBase
 
@@ -29,6 +30,8 @@ class DbMapResultModel(BaseModel, DbMapResultBase):
     """
     # List fields that are aggregated into a string of comma seperated values with basic string splitting on commas
     _csv_list_fields: Set[str] = set()
+    # List field that are json objects
+    _json_fields: Set[str] = set()
     # List fields (type does not matter)
     _list_fields: Set[str] = set()
     # Set fields (type does not matter)
@@ -43,6 +46,18 @@ class DbMapResultModel(BaseModel, DbMapResultBase):
         # Uses the construct method to prevent validation when mapping results
         return cls.construct(*args, **kwargs)
 
+    def _map_json(self, current_dict: dict, record: sqlalchemy.engine.Row, field: str):
+        model_field = self.__fields__[field]
+        if not self._has_been_mapped():
+            try:
+                potential_json_data = record[field]
+                if potential_json_data:
+                    current_dict[field] = json.loads(record[field])
+            except JSONDecodeError as exc:
+                return ErrorWrapper(ValueError(
+                    f'Invalid JSON given to {model_field.alias}', exc), loc=model_field.alias)
+        return None
+
     def _map_list(self, current_dict: dict, record: sqlalchemy.engine.Row, field: str):
         if record[field] is None:
             # Do not map null entries into lists, this may cause problems in the future but it works
@@ -76,17 +91,18 @@ class DbMapResultModel(BaseModel, DbMapResultBase):
     def _map_list_from_string(self, current_dict: dict, record: sqlalchemy.engine.Row, field: str):
         list_string = record[field]
         if list_string and isinstance(list_string, str):
-            values_from_string = list_string.split(',')
+            values_from_string = list(map(str.strip, list_string.split(',')))
 
-            if self._has_been_mapped():
-                # validates the list we are expecting and updates all fields to the expected type or throws error
-                model_field = self.__fields__[field]
-                values, errors_ = model_field.validate(values_from_string, current_dict, loc=model_field.alias)
-                if errors_:
-                    raise ValidationError(errors_, DbMapResultModel)
+            model_field = self.__fields__[field]
+            # pre-validates the list we are expecting because we want to ensure all records are validated
+            values, errors_ = model_field.validate(values_from_string, current_dict, loc=model_field.alias)
+            if errors_:
+                raise ValidationError(errors_, DbMapResultModel)
+
+            if self._has_been_mapped() and current_dict[field]:
                 current_dict[field].extend(values)
             else:
-                current_dict[field] = values_from_string
+                current_dict[field] = values
 
     def map_record(self, record: sqlalchemy.engine.Row) -> None:
         """
@@ -101,12 +117,17 @@ class DbMapResultModel(BaseModel, DbMapResultBase):
           - Remove all DB fields that are present in _dict_value_mappings since they were likely added above
         :param record: the DB record
         """
+        errors = []
         current_dict: dict = self.__dict__
         for field in record.keys():
             if field in self._list_fields:
                 self._map_list(current_dict, record, field)
             elif field in self._csv_list_fields:
                 self._map_list_from_string(current_dict, record, field)
+            elif field in self._json_fields:
+                error = self._map_json(current_dict, record, field)
+                if error:
+                    errors.append(error)
             elif field in self._set_fields:
                 self._map_set(current_dict, record, field)
             elif field in self._dict_key_fields:
@@ -116,6 +137,8 @@ class DbMapResultModel(BaseModel, DbMapResultBase):
                 if not self._has_been_mapped():
                     current_dict[field] = record[field]
 
+        if errors:
+            raise ValidationError(errors, DbMapResultModel)
         # Remove all dict value fields (if present)
         for db_field in self._dict_value_mappings.values():
             current_dict.pop(db_field, None)
